;; -*- mode: emacs-lisp -*-
(unless (require 'guix-emacs nil 'noerror)
 (package-initialize))
(unless (require 'guix-emacs nil 'noerror)
 ;; package archives
 (when (>= emacs-major-version 24)
   (require 'package)
   (setq package-archives
       '(("GNU_ELPA"     . "https://elpa.gnu.org/packages/")
         ("org"          . "https://orgmode.org/elpa/")
         ("MELPA_Stable" . "https://stable.melpa.org/packages/")
         ("MELPA"        . "https://melpa.org/packages/"))
       package-archive-priorities
       '(("GNU_ELPA"     . 15)
         ("org"          . 10)
 	("MELPA_Stable" . 5)
         ("MELPA"        . 0)))))
(setq visible-bell 1)
(setq inhibit-splash-screen t)
(setq tool-bar-mode nil)
;; window resizing bindings
(global-set-key [C-iso-lefttab] 'shrink-window-horizontally)
(global-set-key [C-tab] 'enlarge-window-horizontally)
(global-set-key (kbd "C-^") 'shrink-window)
(global-set-key (kbd "C-ì") 'enlarge-window)
(when (fboundp 'windmove-default-keybindings)
	(windmove-default-keybindings 'meta))
(set-default-coding-systems 'utf-8)
(set-terminal-coding-system 'utf-8)
(delete-selection-mode)
;(defvar myfont "DejaVu Sans Mono-10" "My preferred font.")
(add-to-list 'default-frame-alist '(font . "DejaVu Sans Mono-10"))
(set-face-attribute 'default t :font "DejaVu Sans Mono-10")
; Liberation Mono Regular-11
; Monospace Regular -11
(load-theme 'tango-dark t)
(unless (require 'guix-emacs nil 'noerror)
 (use-package emojify
   :ensure t
   :pin "GNU_ELPA"))
;; (if (display-graphic-p) 
;;       (setq emojify-display-style 'image)
;;       (setq emojify-display-style 'unicode))
(add-hook 'after-init-hook #'global-emojify-mode)
;;(global-emojify-mode 1)
(put 'dired-find-alternate-file 'disabled nil)
(unless (require 'guix-emacs nil 'noerror)
 (use-package tramp
   :ensure t
   :pin "GNU_ELPA"))
(require 'tramp)
(unless (require 'guix-emacs nil 'noerror)
 (setq tramp-remote-path
       (append tramp-remote-path
        '(tramp-own-remote-path)))
 (setq tramp-remote-path
       (append tramp-remote-path
        '("~/.guix-profile/bin" "~/.guix-profile/sbin"
 	 "/run/current-system/profile/bin" "/run/current-system/profile/sbin"))))
(setq epa-pinentry-mode 'loopback) ;;GPGAssistant config
(pinentry-start)
(require 'epa-file)
(setq delete-by-moving-to-trash t)
(unless (require 'guix-emacs nil 'noerror)
 (use-package trashed   
   :ensure t
   :pin "MELPA"))
(require 'trashed)
(global-set-key "\C-T" 'trashed)
(setq dired-dwim-target t)
(setq dired-listing-switches "-alh")
(unless (require 'guix-emacs nil 'noerror)
 (use-package eshell-bookmark
   :ensure t))
(add-hook 'eshell-mode-hook 'eshell-bookmark-setup)
(unless (require 'guix-emacs nil 'noerror)
 (use-package editorconfig
   :ensure t
   :pin "MELPA_Stable"))
(require 'editorconfig)
(editorconfig-mode 1)
(unless (require 'guix-emacs nil 'noerror)
 (use-package disk-usage
   :ensure t))
(add-hook 'sql-interactive-mode-hook
          (lambda ()
            (toggle-truncate-lines t)))
(require 'auth-source-pass)
(auth-source-pass-enable)
(unless (require 'guix-emacs nil 'noerror)
 (use-package password-store
   :ensure t
   :pin "MELPA_Stable"))
(unless (require 'guix-emacs nil 'noerror)
 (use-package pass
   :ensure t
   :pin "MELPA_Stable"))
(unless (require 'guix-emacs nil 'noerror)
 (use-package password-generator
   :ensure t
   :pin "MELPA_Stable"))
(unless (require 'guix-emacs nil 'noerror)
 (use-package auctex
   :ensure t
   :pin "GNU_ELPA"))
(require 'tex)
(setq TeX-auto-save t)
(setq TeX-parse-self t)
(setq-default TeX-master nil)
(add-hook 'LaTeX-mode-hook 'turn-on-reftex)
(setq reftex-plug-into-AUCTeX t)
(add-hook 'LaTeX-mode-hook 'visual-line-mode)
(add-hook 'LaTeX-mode-hook 'flyspell-mode)
(add-hook 'LaTeX-mode-hook 'LaTeX-math-mode)
(unless (require 'guix-emacs nil 'noerror)
 (use-package markdown-mode
   :ensure t
   :pin "MELPA_Stable"))
(add-to-list 'auto-mode-alist
             '("\\.\\(?:md\\|markdown\\|mkd\\|mdown\\|mkdn\\|mdwn\\)\\'" . markdown-mode))
(add-to-list 'auto-mode-alist '("README\\.md\\'" . gfm-mode))
(unless (require 'guix-emacs nil 'noerror)
 (use-package dockerfile-mode
   :ensure t
   :pin "MELPA_Stable"))
(add-to-list 'auto-mode-alist '("Dockerfile\\'" . dockerfile-mode))
(unless (require 'guix-emacs nil 'noerror)
 (use-package docker-compose-mode
   :ensure t
   :pin "MELPA_Stable"))
(unless (require 'guix-emacs nil 'noerror)
 (use-package csv-mode
   :ensure t
   :pin "GNU_ELPA"))
(add-to-list 'auto-mode-alist '("\\.[Cc][Ss][Vv]\\'" . csv-mode))
(autoload 'csv-mode "csv-mode"
  "Major mode for editing comma-separated value files." t)
(unless (require 'guix-emacs nil 'noerror)
 (use-package lorem-ipsum
   :ensure t
   :pin "MELPA_Stable"))
(unless (require 'guix-emacs nil 'noerror)
 (use-package nov
   :ensure t
   :pin "MELPA_Stable"))
(require 'nov)
(add-to-list 'auto-mode-alist '("\\.epub\\'" . nov-mode))
(defun my-nov-font-setup ()
  (face-remap-add-relative 'variable-pitch :family "Liberation Serif"
                                           :height 1.3))
(add-hook 'nov-mode-hook 'my-nov-font-setup)
(setq nov-text-width most-positive-fixnum)
(add-hook 'nov-mode-hook 'visual-line-mode)
(unless (require 'guix-emacs nil 'noerror)
 (use-package magit
   :ensure t
   :pin "MELPA_Stable"))
(global-set-key (kbd "C-x g") 'magit-status)
(setq mml-secure-openpgp-sign-with-sender t)
(setq notmuch-crypto-process-mime t)
(setq message-kill-buffer-on-exit t)
(setq notmuch-saved-searches 
  '((:name "unread" :query "tag:unread not (Precedence:list or Precedence:bulk)" :key "u")
    (:name "all mail" :query "*" :key "a")))
(require 'notmuch-show)
(define-key notmuch-show-mode-map "t"
  (lambda()
    "Capture mail to org mode as TODO item."
    (interactive)
    (org-store-link nil)
    (org-capture nil "m")))
(unless (require 'guix-emacs nil 'noerror)
 (use-package gnus-alias
   :ensure t
   :pin "MELPA_Stable"))
(require 'gnus-alias)
(gnus-alias-init)
(setq notmuch-fcc-dirs
      '(("REDACTED" . ".INBOX.private -inbox -unread +sent")
	(".*" . ". -inbox +sent -unread")))

;; my identities
;; (setq gnus-alias-identity-alist
;;       '(("REDACTED"
;;
;; [REDACTED]
;;
;; 	 )))

;; default identity
(setq gnus-alias-default-identity "REDACTED")

;; Define rules to match identities
;; WARNING: header symbol "to" does non work, use "any"!
;; (setq gnus-alias-identity-rules
;;   (quote

;; [REDACTED]

(unless (require 'guix-emacs nil 'noerror)
 (use-package org
   :mode (("\\.org$" . org-mode))
   :ensure org-plus-contrib
   :pin org))
(global-set-key (kbd "\C-cl") 'org-store-link)
(global-set-key (kbd "\C-cb") 'org-iswitchb)
(global-set-key	(kbd "\C-ca") 'org-agenda)
(global-set-key	(kbd "\C-cc") 'org-capture)
(require 'org-id)
(require 'org-toc)
(require 'org-tempo)
(setq org-confirm-babel-evaluate nil)
(org-babel-do-load-languages
 'org-babel-load-languages
 '((calc . t)
   (haskell . t)
   (ditaa . t)
   (shell . t)))
(setq org-directory "~/.org")

(when (file-exists-p "~/.org")
 (setq org-agenda-files (quote ("~/.org"
                                "~/.org/customers"))))
(setq org-log-done t)
(require 'org-capture)
(setq org-default-notes-file "~/.org/gtd.org")
(setq org-capture-templates nil)
(add-to-list 'org-capture-templates
  '("t" "Todo" 
    entry (file+headline "~/.org/gtd.org" "Tasks")
    "* TODO %?\n  %i\n  %a"))
(require 'ol-notmuch)
(add-to-list 'org-capture-templates
  '("m" "Todo from email"
    entry (file+headline "~/.org/gtd.org" "Tasks")
    "* TODO %:subject\n SCHEDULED: %t\n :PROPERTIES:\n :END:\n\n %a\n\n %i\n %?"))
(add-to-list 'org-capture-templates
  '("j" "Journal" 
    entry (file+olp+datetree "~/.org/journal.org")
    "* %?\nEntered on %U\n  %i\n  %a"))
(setq org-refile-use-outline-path 'file)
(setq org-refile-targets '((nil :maxlevel . 1) (org-agenda-files :maxlevel . 1)))
(require 'org-crypt)
(org-crypt-use-before-save-magic)
(setq org-crypt-key "D37D0EA7CECC3912")
(setq auto-save-default nil)
(setq org-tags-exclude-from-inheritance (quote ("crypt")))
(unless (require 'guix-emacs nil 'noerror)
 (use-package org-noter
  :ensure org-noter))
(require 'org-noter)
(unless (require 'guix-emacs nil 'noerror)
 (use-package org-ref
  :ensure org-ref))
(require 'org-ref)
(setq org-ref-notes-directory "~/.org/notes"
      org-ref-bibliography-notes "~/.org/biblio/index.org"
      org-ref-default-bibliography '("~/.org/biblio/index.bib")
      org-ref-pdf-directory "~/.org/lib/")
(require 'org-protocol)
(add-to-list 'org-modules 'org-eww)
;; FIXME: no way to get URL when capturing from eww
;(defun url()
;(plist-get org-store-link-plist :url))
(add-to-list 'org-capture-templates
  '("p" "Web page selection"
    entry (file "~/.org/web-annotations.org")
    "* %:url    %^g\nCaptured on: %U\n\n#+BEGIN_QUOTE\n%i\n#+END_QUOTE\n\n\n%?"))
(add-to-list 'org-capture-templates
  '("L" "Web page link"
    entry (file "~/.org/web-annotations.org")
    "* %:url    %^g\nCaptured on: %U\n\n\n%?"))
(unless (require 'guix-emacs nil 'noerror)
 (use-package org-web-tools
   :ensure t
   :pin "MELPA"))
(require 'org-web-tools)
(add-to-list 'org-capture-templates
  '("w" "Org-web-tools entire page capture"
    entry (file "~/.org/web-annotations.org")
    "%(org-web-tools--url-as-readable-org \"%:link\")"))

;; FIXME: does not work
(add-to-list 'org-capture-templates
  '("s" "Org-web-tools selection capture"
    entry (file "~/.org/web-annotations.org")
    "%(org-web-tools--html-to-org-with-pandoc \"%i\")"))
;; FIXME: missing in Guix
;;(use-package ob-http
;; :ensure ob-ohttp
;;)
(setq browse-url-browser-function 'eww-browse-url)
(defun xah-rename-eww-hook ()
  "Rename eww browser's buffer so sites open in new page."
  (rename-buffer "eww" t))
(add-hook 'eww-mode-hook #'xah-rename-eww-hook)

;; C-u M-x eww will force a new eww buffer
;; snippet from http://emacs.stackexchange.com/a/24477
(defun modi/force-new-eww-buffer (orig-fun &rest args)
  "When prefix argument is used, a new eww buffer will be created,
regardless of whether the current buffer is in `eww-mode'."
  (if current-prefix-arg
      (with-temp-buffer
        (apply orig-fun args))
    (apply orig-fun args)))  
(advice-add 'eww :around #'modi/force-new-eww-buffer)
(unless (require 'guix-emacs nil 'noerror)
 (use-package elfeed
   :ensure t
   :pin "MELPA")
 (use-package elfeed-org
   :ensure t
   :pin "MELPA"))
(require 'elfeed-org)
(elfeed-org)
;; Optionally specify a number of files containing elfeed
;; configuration. If not set then the location below is used.
(setq rmh-elfeed-org-files (list "~/.emacs.d/elfeed.org"))
(global-set-key (kbd "C-x w") 'elfeed)
(setq-default elfeed-search-filter "+unread ")
(add-hook 'elfeed-new-entry-hook
          (elfeed-make-tagger :before "9 weeks ago"
                              :remove 'unread))
(unless (require 'guix-emacs nil 'noerror)
 (use-package telega
   :ensure t
   :pin "MELPA_Stable"))
(add-hook 'telega-load-hook 'telega-mode-line-mode)
(unless (require 'guix-emacs nil 'noerror)
 (use-package docker-tramp
   :ensure t
   :pin "MELPA_Stable"))
(unless (require 'guix-emacs nil 'noerror)
 (use-package docker
   :ensure t
   :pin "MELPA_Stable"))
(global-set-key (kbd "C-c d") 'docker)
(unless (require 'guix-emacs nil 'noerror)
 (use-package guix
   :ensure t
   :pin "MELPA_Stable"))
(eval-after-load "dired-aux"
   '(add-to-list 'dired-compress-file-suffixes 
                 '("\\.zip\\'" ".zip" "unzip")))
